export class VertexBuffer{
    readonly id : WebGLBuffer;

    constructor(id : WebGLBuffer){
        this.id = id;
    }
    bind(gl:WebGL2RenderingContext){
        gl.bindBuffer(gl.ARRAY_BUFFER,this.id);
    }
    unbind(gl:WebGL2RenderingContext){
        gl.bindBuffer(gl.ARRAY_BUFFER,null);
    }
}

/***
 * create vertex buffer
 * @param gl GLContext
 * @param data vertex data to draw
 * @example ```
 * const buffer = createVertexBuffer(gl,[
 * -0.5,0.5,
 * 0.5,0.5,
 * 0.5,-0.5,
 * -0.5,-0.5]);
 * ```
 */
export function createVertexBuffer(gl:WebGL2RenderingContext, data:number[]): VertexBuffer{
    const id = gl.createBuffer();
    gl.bindBuffer(gl.ARRAY_BUFFER,id);
    gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(data),gl.STATIC_DRAW);
    return new VertexBuffer(id);
}

export class IndexBuffer{
    readonly id: WebGLBuffer;
    readonly count:number;
    constructor(id: WebGLBuffer, count:number){
        this.id = id;
        this.count = count;
    }
    bind(gl:WebGL2RenderingContext){
        gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER,this.id);
    }
    unbind(gl:WebGL2RenderingContext){
        gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER,null);
    }
}

export function createIndexBuffer(gl:WebGL2RenderingContext, data:number[]){
    const id = gl.createBuffer();
    gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER,id);
    gl.bufferData(gl.ELEMENT_ARRAY_BUFFER,new Uint16Array(data),gl.STATIC_DRAW);
    return new IndexBuffer(id,data.length);
}