import { RenderProgram } from "./program";
import {createProgramFromSource, ProgramError, ShaderError} from "./gl_util";
import { createIndexBuffer, createVertexBuffer, IndexBuffer, VertexBuffer } from "./glWrapper";

const vertex_shader_code = `#version 300 es
layout(location=0) in vec4 pos;

void main() {
  gl_Position = pos;
}
`;

const frag_shader_code = `#version 300 es
precision highp float;
layout(location=0) out vec4 outColor;
void main() {
  outColor = vec4(1.0,0.5,1.0,1.0);
}
`;

export class TriangleRenderer implements RenderProgram{
    readonly program : WebGLProgram;
    vao : WebGLVertexArrayObject;
    indexBuffer : IndexBuffer;
    constructor(gl: WebGL2RenderingContext){try{
        this.program = createProgramFromSource(gl,vertex_shader_code,frag_shader_code);
        //gl.useProgram(this.program);
    }
    catch(e){
        if(e instanceof ShaderError){
            console.log(e.message,"\n",e.info);
        }
        else if(e instanceof ProgramError){
            console.log(e.message,"\n",e.info);
        }
        else throw e;
    }
    }
    prepare(gl : WebGL2RenderingContext){
        const vertex = createVertexBuffer(gl,[
            0.5,0.5,
            0.0,0.25,
            0.0,0.75]);
        this.indexBuffer = createIndexBuffer(gl,[0,1,2]);
        const posLoc = gl.getAttribLocation(this.program,"pos");
        this.vao = gl.createVertexArray();
        gl.bindVertexArray(this.vao);
        gl.enableVertexAttribArray(posLoc);
        vertex.bind(gl);
        gl.vertexAttribPointer(posLoc,2,gl.FLOAT,false,0,0);
        this.indexBuffer.bind(gl);
    }
    draw(gl: WebGL2RenderingContext): void {
        gl.useProgram(this.program);
        gl.bindVertexArray(this.vao);
        this.indexBuffer.bind(gl);
        gl.drawElements(gl.TRIANGLES,this.indexBuffer.count,gl.UNSIGNED_SHORT,0);
    }
}